@@ -31,9 +31,26 @@ keytype_to_algo (const gchar *salgo)
 		return GCRY_PK_RSA;
 	else if (strcmp (salgo, "ssh-dss") == 0)
 		return GCRY_PK_DSA;
+	else if ((strcmp (salgo, "ecdsa-sha2-nistp256") == 0)
+	    || (strcmp (salgo, "ecdsa-sha2-nistp384") == 0)
+	    || (strcmp (salgo, "ecdsa-sha2-nistp521") == 0))
+		return GCRY_PK_ECC;
 	return 0;
 }
 
+static const gchar *
+curve_to_gcry (const gchar *salgo)
+{
+	g_return_val_if_fail (salgo, 0);
+	if (strcmp (salgo, "nistp256") == 0)
+		return "NIST P-256";
+	else if (strcmp (salgo, "nistp384") == 0)
+		return "NIST P-384";
+	else if (strcmp (salgo, "nistp521") == 0)
+		return "NIST P-521";
+	return NULL;
+}
+
 static gboolean
 read_mpi (EggBuffer *req, gsize *offset, gcry_mpi_t *mpi)
 {
@@ -85,6 +102,40 @@ read_public_dsa (EggBuffer *req, gsize *offset, gcry_sexp_t *sexp)
 	return TRUE;
 }
 
+#define SEXP_PUBLIC_ECDSA  \
+	"(public-key"      \
+	"  (ecdsa"         \
+	"    (curve %s)"   \
+	"    (q %b)))"
+
+static gboolean
+read_public_ecdsa (EggBuffer *req, gsize *offset, gcry_sexp_t *sexp)
+{
+	int gcry;
+	const gchar *curve_name = NULL;
+	const guchar *q = NULL;
+	size_t q_len;
+	gchar *curve = NULL;
+
+	if (!egg_buffer_get_string (req, *offset, offset, &curve, (EggBufferAllocator)g_realloc))
+		return FALSE;
+
+	if (!egg_buffer_get_byte_array (req, *offset, offset, &q, &q_len))
+		return FALSE;
+
+	curve_name = curve_to_gcry (curve);
+	g_return_val_if_fail (curve_name, FALSE);
+
+	gcry = gcry_sexp_build (sexp, NULL, SEXP_PUBLIC_ECDSA,
+                                curve_name, q_len, q);
+	if (gcry) {
+		g_warning ("couldn't parse incoming public ECDSA key: %s", gcry_strerror (gcry));
+		return FALSE;
+	}
+
+	return TRUE;
+}
+
 #define SEXP_PUBLIC_RSA  \
 	"(public-key"    \
 	"  (rsa"         \
@@ -139,6 +190,9 @@ read_public (EggBuffer *req, gsize *offset, gcry_sexp_t *key, int *algo)
 	case GCRY_PK_DSA:
 		ret = read_public_dsa (req, offset, key);
 		break;
+	case GCRY_PK_ECC:
+		ret = read_public_ecdsa (req, offset, key);
+		break;
 	default:
 		g_assert_not_reached ();
 		return FALSE;
@@ -155,7 +209,7 @@ read_public (EggBuffer *req, gsize *offset, gcry_sexp_t *key, int *algo)
 }
 
 static GkmDataResult
-load_encrypted_key (EggBytes *data,
+load_encrypted_key (GBytes *data,
                     const gchar *dekinfo,
                     const gchar *password,
                     gssize n_password,
@@ -163,7 +217,7 @@ load_encrypted_key (EggBytes *data,
 {
 	guchar *decrypted = NULL;
 	gsize n_decrypted = 0;
-	EggBytes *bytes;
+	GBytes *bytes;
 	GkmDataResult ret;
 	gint length;
 
@@ -178,11 +232,11 @@ load_encrypted_key (EggBytes *data,
 	if (length > 0)
 		n_decrypted = length;
 
-	bytes = egg_bytes_new_with_free_func (decrypted, n_decrypted, egg_secure_free, decrypted);
+	bytes = g_bytes_new_with_free_func (decrypted, n_decrypted, egg_secure_free, decrypted);
 
 	/* Try to parse */
 	ret = gkm_data_der_read_private_key (bytes, skey);
-	egg_bytes_unref (bytes);
+	g_bytes_unref (bytes);
 
 	if (ret != GKM_DATA_UNRECOGNIZED)
 		return ret;
@@ -195,23 +249,27 @@ is_private_key_type (GQuark type)
 {
 	static GQuark PEM_RSA_PRIVATE_KEY;
 	static GQuark PEM_DSA_PRIVATE_KEY;
+	static GQuark PEM_ECDSA_PRIVATE_KEY;
 	static gsize quarks_inited = 0;
 
 	/* Initialize the first time through */
 	if (g_once_init_enter (&quarks_inited)) {
 		PEM_RSA_PRIVATE_KEY = g_quark_from_static_string ("RSA PRIVATE KEY");
 		PEM_DSA_PRIVATE_KEY = g_quark_from_static_string ("DSA PRIVATE KEY");
+		PEM_ECDSA_PRIVATE_KEY = g_quark_from_static_string ("EC PRIVATE KEY");
 		g_once_init_leave (&quarks_inited, 1);
 	}
 
 	/* Only handle SSHv2 private keys */
-	return (type == PEM_RSA_PRIVATE_KEY || type == PEM_DSA_PRIVATE_KEY);
+	return (type == PEM_RSA_PRIVATE_KEY ||
+		type == PEM_DSA_PRIVATE_KEY ||
+		type == PEM_ECDSA_PRIVATE_KEY);
 }
 
 static void
 parsed_pem_block (GQuark type,
-                  EggBytes *data,
-                  EggBytes *outer,
+                  GBytes *data,
+                  GBytes *outer,
                   GHashTable *headers,
                   gpointer user_data)
 {
@@ -241,8 +299,8 @@ parsed_pem_block (GQuark type,
 
 static void
 digest_pem_block (GQuark type,
-                  EggBytes *data,
-                  EggBytes *outer,
+                  GBytes *data,
+                  GBytes *outer,
                   GHashTable *headers,
                   gpointer user_data)
 {
@@ -258,8 +316,8 @@ digest_pem_block (GQuark type,
 		return;
 
 	*result = g_compute_checksum_for_data (G_CHECKSUM_SHA1,
-	                                       egg_bytes_get_data (data),
-	                                       egg_bytes_get_size (data));
+	                                       g_bytes_get_data (data, NULL),
+	                                       g_bytes_get_size (data));
 }
 
 /* ------------------------------------------------------------------------------
@@ -373,7 +431,7 @@ gkm_ssh_openssh_parse_public_key (gconstpointer input, gsize n_data,
 }
 
 GkmDataResult
-gkm_ssh_openssh_parse_private_key (EggBytes *data,
+gkm_ssh_openssh_parse_private_key (GBytes *data,
                                    const gchar *password,
                                    gssize n_password,
                                    gcry_sexp_t *sexp)
@@ -401,7 +459,7 @@ gkm_ssh_openssh_parse_private_key (EggBytes *data,
 }
 
 gchar *
-gkm_ssh_openssh_digest_private_key (EggBytes *data)
+gkm_ssh_openssh_digest_private_key (GBytes *data)
 {
 	gchar *result = NULL;
 	egg_armor_parse (data, digest_pem_block, &result);
